--- a/nhrpd/nhrp_route.c
+++ b/nhrpd/nhrp_route.c
@@ -51,6 +51,35 @@
 	}
 }
 
+static void nhrp_route_update_delete(struct prefix *p, union sockunion *via,
+		struct interface *ifp)
+{
+	struct route_node *rn;
+	struct route_info *ri;
+	unsigned int n = 0;
+
+	rn = nhrp_route_update_get(p, 0);
+	if (!rn)
+		return;
+	if (!(ri = rn->info))
+		goto route_delete;
+	if (!(ri->ifp) && !(ri->nhrp_ifp) &&
+	    	sockunion_family(&ri->via) == AF_UNSPEC)
+		goto route_delete;
+	if (ifp && ri->ifp && ifp->ifindex != ri->ifp->ifindex)
+		return;
+	if (via && !sockunion_same(&ri->via, via))
+		return;
+
+route_delete:
+	n = rn->lock;
+	if (ri)
+		XFREE(MTYPE_NHRP_ROUTE, rn->info);
+	while (n--)
+		route_unlock_node(rn);
+	return;
+}
+
 static void nhrp_route_update_put(struct route_node *rn)
 {
 	struct route_info *ri = rn->info;
@@ -240,6 +269,8 @@
 
 	nhrp_route_update_zebra(&api.prefix, &nexthop_addr, added ? ifp : NULL);
 	nhrp_shortcut_prefix_change(&api.prefix, !added);
+	if (!added)
+		nhrp_route_update_delete(&api.prefix, &nexthop_addr, ifp);
 
 	return 0;
 }
